@@ -1,7 +1,7 @@
 import sys
 import types
 
-class Cursor(object):
+class BaseCursor(object):
     """
     A base for Cursor classes. Useful attributes:
 
@@ -13,58 +13,26 @@ class Cursor(object):
         default number of rows fetchmany() will fetch
     """
 
-    def __init__(self, conn):
-        self._cs = conn.connection.cursor()
+    def __init__(self, _cs):
+        self._cs = _cs
         self.arraysize = 1
         self.rowcount = -1
         self.description = None
-        
-        self.charset = conn.charset
-        self._cs._set_charset_name(conn.charset)
 
     def __del__(self):
         self.close()
 
     def close(self):
         """Close the cursor. No further queries will be possible."""
-        try:
-            self._cs.close()
-            self._cs = None
-        except AttributeError:   # self._cs not exists
-            pass
+        self._cs.close()
 
     def _bind_params(self, args):
-        if type(args) not in (tuple, list):
-            args = [args,]
         args = list(args)
         for i in range(len(args)):
-            if args[i] == None:
-                pass
-            elif isinstance(args[i], bool):
-                if args[i] == True:
-                    args[i] = '1'
-                else:
-                    args[i] = '0'
-            else:
-                # Python3.X dosen't support unicode keyword.
-                try:
-                    mytest = unicode
-                except NameError:
-                    if isinstance(args[i], str):
-                        pass
-                    elif isinstance(args[i], bytes):
-                        args[i] = args[i].decode(self.charset)
-                    else:
-                        args[i] = str(args[i])
-                else:
-                    if isinstance(args[i], unicode):
-                        args[i] = args[i].encode(self.charset)
-                    else:
-                        args[i] = str(args[i])
-
+            args[i] = str(args[i])
             self._cs.bind_param(i+1, args[i])
 
-    def execute(self, query, args=None):
+    def execute(self, query, *args):
         """
         Execute a query.
         
@@ -73,10 +41,12 @@ class Cursor(object):
 
         Returns long integer rows affected, if any
         """
-        self._cs.prepare(query)
+        if len(args) == 1 and type(args[0]) in (tuple, list):
+            args = args[0]
 
-        if args != None:
-            self._bind_params(args)
+        self._cs.prepare(query)
+        
+        self._bind_params(args)
 
         r = self._cs.execute()
         self.rowcount = self._cs.rowcount
@@ -101,9 +71,11 @@ class Cursor(object):
         for p in args:
             self.execute(query, *(p,))
 
+    def _fetch_row(self):
+        return self._cs.fetch_row(self._fetch_type)
     
     def fetchone(self):
-        return self._cs.fetch_row()
+        return self._fetch_row()
 
     def _fetch_many(self, size):
         rlist = []
@@ -152,19 +124,22 @@ class Cursor(object):
         """
         pass
 
-    def __iter__(self):
-        """
-        Iteration over the result set which calls self.fetchone()
-        and returns the next row.
-        """
-        return self  # iter(self.fetchone, None)
+class CursorTupleRowsMixIn(object):
 
-    def next(self):
-        return self.__next__()
+    _fetch_type = 0
 
-    def __next__(self):
-        row = self.fetchone()
-        if row is None:
-            raise StopIteration
-        return row
+class CursorDictTupleMixIn(object):
 
+    _fetch_type = 1
+
+class Cursor(CursorTupleRowsMixIn, BaseCursor):
+    '''
+    This is the standard Cursor class that returns rows as tuples
+    and stores the result set in the client.
+    '''
+
+class DictCursor(CursorDictTupleMixIn, BaseCursor):
+    '''
+    This is a Cursor class that returns rows as dictionaries and
+    stores the result set in the client.
+    '''
