@@ -18,12 +18,16 @@
 #define CUBRID_ER_MSG_LEN 1024
 
 static PyObject *_cubrid_error;
+static PyObject *_cubrid_warning;
 static PyObject *_cubrid_interface_error;
 static PyObject *_cubrid_database_error;
+static PyObject *_cubrid_internal_error;
+static PyObject *_cubrid_operational_error;
+static PyObject *_cubrid_programming_error;
+static PyObject *_cubrid_integrity_error;
+static PyObject *_cubrid_data_error;
 static PyObject *_cubrid_not_supported_error;
 
-static PyObject *_func_Decimal;
-
 static struct _cubrid_isolation
 {
   int level;
@@ -65,54 +69,43 @@ static struct _error_message
 } cubrid_err_msgs[] =
 {
   {
-  -2001, "Memory allocation error"},
+  CUBRID_ER_NO_MORE_MEMORY, "Memory allocation error"},
   {
-  -2002, "Invalid API call"},
+  CUBRID_ER_INVALID_SQL_TYPE, "Invalid API call"},
   {
-  -2003, "Cannot get column info"},
+  CUBRID_ER_CANNOT_GET_COLUMN_INFO, "Cannot get column info"},
   {
-  -2004, "Array initializing error"},
+  CUBRID_ER_INIT_ARRAY_FAIL, "Array initializing error"},
   {
-  -2005, "Unknown column type"},
+  CUBRID_ER_UNKNOWN_TYPE, "Unknown column type"},
   {
-  -2006, "Invalid parameter"},
+  CUBRID_ER_INVALID_PARAM, "Invalid parameter"},
   {
-  -2007, "Invalid array type"},
+  CUBRID_ER_INVALID_ARRAY_TYPE, "Invalid array type"},
   {
-  -2008, "Invalid type"},
+  CUBRID_ER_NOT_SUPPORTED_TYPE, "Invalid type"},
   {
-  -2009, "File open error"},
+  CUBRID_ER_OPEN_FILE, "File open error"},
   {
-  -2010, "Temporary file open error"},
+  CUBRID_ER_CREATE_TEMP_FILE, "Temporary file open error"},
   {
-  -2012, "Invalid cursor position"},
+  CUBRID_ER_INVALID_CURSOR_POS, "Invalid cursor position"},
   {
-  -2013, "SQL statement not prepared"},
+  CUBRID_ER_SQL_UNPREPARE, "SQL statement not prepared"},
   {
-  -2014, "Some parameter not binded"},
+  CUBRID_ER_PARAM_UNBIND, "Some parameter not binded"},
   {
-  -2015, "Invalid schema type"},
+  CUBRID_ER_SCHEMA_TYPE, "Invalid schema type"},
   {
-  -2016, "Can not read file"},
+  CUBRID_ER_READ_FILE, "Can not read file"},
   {
-  -2017, "Can not write file"},
+  CUBRID_ER_WRITE_FILE, "Can not write file"},
   {
-  -2018, "LOB not exist"},
+  CUBRID_ER_LOB_NOT_EXIST, "LOB not exist"},
   {
   0, ""}
 };
 
-#if (PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION <= 4)
-typedef int Py_ssize_t;
-#endif
-
-static PyObject *
-_cubrid_return_PyUnicode_FromString (const char *buf, Py_ssize_t size, 
-        const char *encoding, const char *errors)
-{
-  return PyUnicode_Decode (buf, size, encoding, errors);
-}
-
 static PyObject *
 _cubrid_return_PyString_FromString (const char *buf)
 {
@@ -138,7 +131,7 @@ get_error_msg (int err_code, char *err_msg)
 {
   int i;
 
-  if (err_code >= -2000)
+  if (err_code > CCI_ER_END)
     {
       return cci_get_err_msg (err_code, err_msg, CUBRID_ER_MSG_LEN);
     }
@@ -159,13 +152,11 @@ get_error_msg (int err_code, char *err_msg)
 PyObject *
 handle_error (int e, T_CCI_ERROR * error)
 {
-  PyObject *t, *exception = NULL;
+  PyObject *t;
   int err_code;
   char msg[CUBRID_ER_MSG_LEN] = { '0' };
   char err_msg[CUBRID_ER_MSG_LEN] = { '\0' }, *facility_msg;
 
-  exception = _cubrid_error;
-
   if (e == CCI_ER_DBMS)
     {
       facility_msg = "DBMS";
@@ -173,34 +164,30 @@ handle_error (int e, T_CCI_ERROR * error)
 	{
 	  err_code = error->err_code;
 	  snprintf (err_msg, CUBRID_ER_MSG_LEN, "%s", error->err_msg);
-          exception = _cubrid_database_error;
 	}
       else
 	{
 	  err_code = 0;
 	  snprintf (err_msg, CUBRID_ER_MSG_LEN, "Unknown DBMS Error");
-          exception = _cubrid_not_supported_error;
 	}
     }
   else
     {
-      exception = _cubrid_interface_error;
-
       if (get_error_msg (e, err_msg) < 0)
 	{
 	  snprintf (err_msg, CUBRID_ER_MSG_LEN, "Unknown Error");
 	}
       err_code = e;
 
-      if (e > -1000)
-	{
-	  facility_msg = "CCI";
-	}
-      else if (e > -2000)
+      if (e > CAS_ER_IS)
 	{
 	  facility_msg = "CAS";
 	}
-      else if (e > -3000)
+      else if (e > CCI_ER_END)
+	{
+	  facility_msg = "CCI";
+	}
+      else if (e > CUBRID_ER_END)
 	{
 	  facility_msg = "CLIENT";
 	}
@@ -216,10 +203,10 @@ handle_error (int e, T_CCI_ERROR * error)
   if (!(t = PyTuple_New (2)))
     return NULL;
 
-  PyTuple_SetItem (t, 0, _cubrid_return_PyInt_FromLong ((long) err_code));
+  PyTuple_SetItem (t, 0, _cubrid_return_PyInt_FromLong ((long) e));
   PyTuple_SetItem (t, 1, _cubrid_return_PyString_FromString (msg));
 
-  PyErr_SetObject (exception, t);
+  PyErr_SetObject (_cubrid_error, t);
   Py_DECREF (t);
 
   return NULL;
@@ -274,6 +261,82 @@ static char _cubrid_escape_string__doc__[] =
 "escape_string()\n\
 Escape special characters in a string for use in an SQL statement";
 
+#ifdef _WIN32
+static PyObject *
+_cubrid_escape_string (PyObject * self, PyObject *args,
+                       PyObject * kwargs)
+{
+  static char *kwList[] = {"escape_string", "no_backslash_escapes", NULL};
+  int no_backslash_escapes = -1;
+  char *unescape_string = NULL, *escape_string = NULL;
+  int len = -1, res;
+  T_CCI_ERROR error;
+  
+    PyObject *op;
+
+    int escape_str_len = 0, i = 0;
+    char *s = NULL;
+
+
+  if (!PyArg_ParseTupleAndKeywords (args, kwargs,
+            "s#|i", kwList, &unescape_string, &len, &no_backslash_escapes))
+    {
+      return NULL;
+    }
+	
+    if (len < 0)
+    {
+        return handle_error (CUBRID_ER_INVALID_PARAM, NULL);
+    }
+
+    for (i = 0; i < len; i++) {
+        if (unescape_string[i] == '\'') {
+            escape_str_len++;
+        }
+        escape_str_len++;
+    }
+
+    if (escape_str_len == len) {
+#if PY_MAJOR_VERSION >= 3
+        op = PyUnicode_FromStringAndSize (unescape_string, len);
+#else
+        op = PyString_FromStringAndSize (unescape_string, len);
+#endif
+        return op;
+    }
+
+    escape_string = (char *) malloc (len * 2 + 16);
+
+    if (!escape_string)
+    {
+        return handle_error (CUBRID_ER_NO_MORE_MEMORY, NULL);
+    }
+
+    memset(escape_string, 0, len * 2 + 16);
+
+    s = escape_string;
+    for (i = 0; i < len; i++) {
+        if (unescape_string[i] == '\'') {
+            *s++ = '\'';
+            *s++ = '\'';
+        } else {
+            *s++ = unescape_string[i];
+        }
+    }
+
+    escape_string[escape_str_len] = '\0';
+
+#if PY_MAJOR_VERSION >= 3
+    op = PyUnicode_FromStringAndSize (escape_string, escape_str_len);
+#else
+    op = PyString_FromStringAndSize (escape_string, escape_str_len);
+#endif
+
+    free (escape_string);
+
+    return op;
+}
+#else
 static PyObject *
 _cubrid_escape_string (PyObject * self, PyObject *args,
                        PyObject * kwargs)
@@ -331,6 +394,7 @@ _cubrid_escape_string (PyObject * self, PyObject *args,
 
   return op;
 }
+#endif
 
 static PyObject *
 _cubrid_ConnectionObject_new (PyTypeObject * type, PyObject * args,
@@ -372,12 +436,21 @@ _cubrid_ConnectionObject_init (_cubrid_ConnectionObject * self,
 
   snprintf (buf, 1024, "cci:%s", url);
 
+#ifdef _WIN32
   con = cci_connect_with_url (buf, user, passwd);
   if (con < 0)
     {
       handle_error (con, NULL);
       return -1;
     }
+#else
+  con = cci_connect_with_url_ex (buf, user, passwd, &error);
+  if (con < 0)
+    {
+      handle_error (con, &error);
+      return -1;
+    }
+#endif
 
   self->handle = con;
 
@@ -448,7 +521,7 @@ Get the cursor class. Return a new Cursor Object.\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb:::\", \"public\")\n\
+  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb\", \"public\")\n\
   cur = con.cursor()\n\
   ...\n\
   other operations\n\
@@ -491,7 +564,7 @@ Create a large object. Return a new lob object.\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb:::\", \"public\")\n\
+  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb\", \"public\")\n\
   cur = con.cursor()\n\
   cur.prepare('insert into test_lob(image) values (?)')\n\
   lob = con.lob()\n\
@@ -589,7 +662,7 @@ Returns a string that represents the server version number.\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb:::\", \"public\")\n\
+  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb\", \"public\")\n\
   print con.server_version()\n\
   con.close()";
 
@@ -622,7 +695,7 @@ Return a string that represents the CUBRID client library\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb:::\", \"public\")\n\
+  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb\", \"public\")\n\
   print con.client_version()\n\
   con.close()";
 
@@ -654,7 +727,7 @@ mode: string. It will be ON/OFF, TRUE/FALSE, YES/NO\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb:::\", \"public\")\n\
+  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb\", \"public\")\n\
   con.set_autocommit('ON')\n\
   print con.autocommit\n\
   con.close()";
@@ -722,7 +795,7 @@ isolation_level maybe::\n\
 Example::\n\
   import _cubrid\n\
   form _cubrid import *\n\
-  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb:::\", \"public\")\n\
+  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb\", \"public\")\n\
   con.set_isolation_level(CUBRID_REP_CLASS_REP_INSTANCE)\n\
   print con.isolation_level\n\
   con.close()";
@@ -766,7 +839,7 @@ Return values::\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb:::\", \"public\")\n\
+  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb\", \"public\")\n\
   print con.ping()\n\
   con.close()";
 
@@ -845,7 +918,7 @@ columns that were updated by the previous INSERT query.\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb:::\", \"public\")\n\
+  con = _cubrid.connect(\"CUBRID:localhost:33000:demodb\", \"public\")\n\
   cur = con.curosr()\n\
   cur.prepare(\"create table test_cubrid(id NUMERIC\n\
           AUTO_INCREMENT(10300, 1), name VARCHAR(50))\")\n\
@@ -879,9 +952,7 @@ _cubrid_ConnectionObject_last_insert_id (_cubrid_ConnectionObject * self,
   else
     {
       strncpy (ret, name, sizeof (ret) - 1);
-#ifndef _WIN32
       free (name);
-#endif
     }
 
   return _cubrid_return_PyString_FromString (ret);
@@ -938,7 +1009,6 @@ _cubrid_ConnectionObject_schema_to_pyvalue (_cubrid_ConnectionObject * self,
 #endif
 	  Py_DECREF (tmpval);
 	}
-      break;
     default:
       res = cci_get_data (request, index, CCI_A_TYPE_STR, &buffer, &ind);
       if (res < 0)
@@ -1094,7 +1164,7 @@ Return values::\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect('CUBRID:localhost:33000:demodb:::', 'public')\n\
+  con = _cubrid.connect('CUBRID:localhost:33000:demodb', 'public')\n\
   print con.schema_info(CUBRID_SCH_TABLE, 'test_cubrid')\n\
   con.close()";
 
@@ -1188,7 +1258,77 @@ static char _cubrid_ConnectionObject_escape_string__doc__[] =
 "escape_string()\n\
 Escape special characters in a string for use in an SQL statement";
 
+#ifdef _WIN32
+static PyObject *
+_cubrid_ConnectionObject_escape_string (_cubrid_ConnectionObject * self,
+				        PyObject * args)
+{
+    char *unescape_string = NULL, *escape_string = NULL;
+    int len = -1, res;
+    PyObject *op;
+    //T_CCI_ERROR error;
 
+    int escape_str_len = 0, i = 0;
+    char *s = NULL;
+
+    if (!PyArg_ParseTuple (args, "s#", &unescape_string, &len))
+    {
+        return NULL;
+    }
+
+    if (len < 0)
+    {
+        return handle_error (CUBRID_ER_INVALID_PARAM, NULL);
+    }
+
+    for (i = 0; i < len; i++) {
+        if (unescape_string[i] == '\'') {
+            escape_str_len++;
+        }
+        escape_str_len++;
+    }
+
+    if (escape_str_len == len) {
+#if PY_MAJOR_VERSION >= 3
+        op = PyUnicode_FromStringAndSize (unescape_string, len);
+#else
+        op = PyString_FromStringAndSize (unescape_string, len);
+#endif
+        return op;
+    }
+
+    escape_string = (char *) malloc (len * 2 + 16);
+
+    if (!escape_string)
+    {
+        return handle_error (CUBRID_ER_NO_MORE_MEMORY, NULL);
+    }
+
+    memset(escape_string, 0, len * 2 + 16);
+
+    s = escape_string;
+    for (i = 0; i < len; i++) {
+        if (unescape_string[i] == '\'') {
+            *s++ = '\'';
+            *s++ = '\'';
+        } else {
+            *s++ = unescape_string[i];
+        }
+    }
+
+    escape_string[escape_str_len] = '\0';
+
+#if PY_MAJOR_VERSION >= 3
+    op = PyUnicode_FromStringAndSize (escape_string, escape_str_len);
+#else
+    op = PyString_FromStringAndSize (escape_string, escape_str_len);
+#endif
+
+    free (escape_string);
+
+    return op;
+}
+#else
 static PyObject *
 _cubrid_ConnectionObject_escape_string (_cubrid_ConnectionObject * self,
 				        PyObject * args)
@@ -1234,6 +1374,7 @@ _cubrid_ConnectionObject_escape_string (_cubrid_ConnectionObject * self,
 
   return op;
 }
+#endif
 
 static char _cubrid_ConnectionObject_close__doc__[] =
 "close()\n\
@@ -1328,35 +1469,9 @@ _cubrid_CursorObject_init (_cubrid_CursorObject * self, PyObject * args,
   self->row_count = -1;
   self->cursor_pos = 0;
 
-  memset(self->charset, 0, sizeof(self->charset));
-
   return 0;
 }
 
-static char _cubrid_CursorObject__set_charset_name__doc__[] =
-"Only used internally. This function should not be used by user.";
-
-static PyObject *
-_cubrid_CursorObject__set_charset_name (_cubrid_CursorObject * self, PyObject * args)
-{
-  char *charset = NULL;
-
-  if (!PyArg_ParseTuple (args, "s", &charset))
-    {
-      return NULL;
-    }
-
-  if (charset != NULL && *charset != '\0')
-    {
-      snprintf(self->charset, sizeof(self->charset), "%s", charset);
-    }
-
-  Py_INCREF (Py_None);
-  return Py_None;
-
-}
-
-
 static void
 _cubrid_CursorObject_reset (_cubrid_CursorObject * self)
 {
@@ -1380,14 +1495,12 @@ _cubrid_CursorObject_reset (_cubrid_CursorObject * self)
 
 static char _cubrid_CursorObject_prepare__doc__[] =
 "prepare(sql)\n\
-This function creates a prepared statement. A prepared statement is a\n\
-server-side object that can be used to optimize performance. You can \n\
-use this statement effectively to execute repeatedly or to process \n\
-long data. Only a single statement can be used. The SQL statement can \n\
-contain zero or more question mark (?) parameter markers for which real \n\
-values will be substituted when the statement is executed. Add a \n\
-parameter when you bind a value in the VALUES clause of INSERT statement \n\
-or in the WHERE clause.\n\
+This function is sort of API which represents SQL statements compiled\n\
+previously. Accordingly, you can use this statement effectively to\n\
+execute several times repeatedly or to process long data Only a single\n\
+statement can be used and a parameter may put a question mark (?) to\n\
+appropriate area in the SQL statement. Add a parameter when you bind\n\
+a value in the VALUES clause of INSERT statement or in the WHERE clause.\n\
 \n\
 sql: string, the sql statement you want to execute.";
 
@@ -1454,7 +1567,7 @@ _cubrid_CursorObject_bind_param (_cubrid_CursorObject * self, PyObject * args)
       return handle_error (CUBRID_ER_SQL_UNPREPARE, NULL);
     }
 
-  if (!PyArg_ParseTuple (args, "iz", &index, &value))
+  if (!PyArg_ParseTuple (args, "is", &index, &value))
     {
       return NULL;
     }
@@ -1487,7 +1600,7 @@ Parameters::\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect('CUBRID:localhost:33000:demodb:::', 'public')\n\
+  con = _cubrid.connect('CUBRID:localhost:33000:demodb', 'public')\n\
   cur = con.cursor()\n\
   \n\
   cur.prepare('create table test_blob(image BLOB)')\n\
@@ -1553,7 +1666,6 @@ _cubrid_CursorObject_set_description (_cubrid_CursorObject * self)
 {
   PyObject *desc, *item;
   int i;
-  char colName[128] = { '\0' };
   int datatype, precision, scale, nullable;
 
   if (self->col_count == 0)
@@ -1567,8 +1679,11 @@ _cubrid_CursorObject_set_description (_cubrid_CursorObject * self)
 
   for (i = 1; i <= self->col_count; i++)
     {
+      char *colName = NULL;
+
       item = PyTuple_New (7);
-      strcpy (colName, CCI_GET_RESULT_INFO_NAME (self->col_info, i));
+
+      colName = CCI_GET_RESULT_INFO_NAME (self->col_info, i);
       precision = CCI_GET_RESULT_INFO_PRECISION (self->col_info, i);
       scale = CCI_GET_RESULT_INFO_SCALE (self->col_info, i);
       nullable = CCI_GET_RESULT_INFO_IS_NON_NULL (self->col_info, i);
@@ -1639,7 +1754,7 @@ row: int, the column you want to get the information.\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect('CUBRID:localhost:33000:demodb:::', 'public')\n\
+  con = _cubrid.connect('CUBRID:localhost:33000:demodb', 'public')\n\
   cur = con.cursor()\n\
   cur.prepare('select * from test_cubrid')\n\
   cur.execute()\n\
@@ -1656,12 +1771,6 @@ _cubrid_CursorObject_result_info (_cubrid_CursorObject * self,
 {
   PyObject *result, *item;
   int i, j, n = 0, len = 0;
-  char col_name[128] = { '\0' };
-  char real_attr[128] = { '\0' };
-  char class_name[128] = { '\0' };
-  char default_value[128] = { '\0' };
-  int type, precision, scale, not_null, auto_increment, unique_key,
-    primary_key, foreign_key, reverse_index, resverse_unique, shared;
 
   if (!PyArg_ParseTuple (args, "|i", &n))
     {
@@ -1694,49 +1803,43 @@ _cubrid_CursorObject_result_info (_cubrid_CursorObject * self,
 
   for (j = 0; i <= len; i++, j++)
     {
+      char *col_name = NULL, *real_attr = NULL;
+      char *class_name = NULL, *default_value = NULL;
+      int type, precision, scale, not_null, auto_increment, unique_key,
+        primary_key, foreign_key, reverse_index, resverse_unique, shared;
+
       item = PyTuple_New (15);
 
       type = CCI_GET_RESULT_INFO_TYPE (self->col_info, i);
       not_null = CCI_GET_RESULT_INFO_IS_NON_NULL (self->col_info, i);
       scale = CCI_GET_RESULT_INFO_SCALE (self->col_info, i);
       precision = CCI_GET_RESULT_INFO_PRECISION (self->col_info, i);
-      strcpy (col_name, CCI_GET_RESULT_INFO_NAME (self->col_info, i));
-      strcpy (real_attr, CCI_GET_RESULT_INFO_ATTR_NAME (self->col_info, i));
-      strcpy (class_name, CCI_GET_RESULT_INFO_CLASS_NAME (self->col_info, i));
-      strcpy (default_value,
-	      CCI_GET_RESULT_INFO_DEFAULT_VALUE (self->col_info, i));
-      auto_increment =
-	CCI_GET_RESULT_INFO_IS_AUTO_INCREMENT (self->col_info, i);
+      col_name = CCI_GET_RESULT_INFO_NAME (self->col_info, i);
+      real_attr = CCI_GET_RESULT_INFO_ATTR_NAME (self->col_info, i);
+      class_name = CCI_GET_RESULT_INFO_CLASS_NAME (self->col_info, i);
+      default_value = CCI_GET_RESULT_INFO_DEFAULT_VALUE (self->col_info, i);
+      auto_increment = CCI_GET_RESULT_INFO_IS_AUTO_INCREMENT (self->col_info, i);
       unique_key = CCI_GET_RESULT_INFO_IS_UNIQUE_KEY (self->col_info, i);
       primary_key = CCI_GET_RESULT_INFO_IS_PRIMARY_KEY (self->col_info, i);
       foreign_key = CCI_GET_RESULT_INFO_IS_FOREIGN_KEY (self->col_info, i);
-      reverse_index =
-	CCI_GET_RESULT_INFO_IS_REVERSE_INDEX (self->col_info, i);
-      resverse_unique =
-	CCI_GET_RESULT_INFO_IS_REVERSE_UNIQUE (self->col_info, i);
+      reverse_index = CCI_GET_RESULT_INFO_IS_REVERSE_INDEX (self->col_info, i);
+      resverse_unique = CCI_GET_RESULT_INFO_IS_REVERSE_UNIQUE (self->col_info, i);
       shared = CCI_GET_RESULT_INFO_IS_SHARED (self->col_info, i);
 
       PyTuple_SetItem (item, 0, _cubrid_return_PyInt_FromLong (type));
       PyTuple_SetItem (item, 1, _cubrid_return_PyInt_FromLong (not_null));
       PyTuple_SetItem (item, 2, _cubrid_return_PyInt_FromLong (scale));
       PyTuple_SetItem (item, 3, _cubrid_return_PyInt_FromLong (precision));
-      PyTuple_SetItem (item, 4,
-		       _cubrid_return_PyString_FromString (col_name));
-      PyTuple_SetItem (item, 5,
-		       _cubrid_return_PyString_FromString (real_attr));
-      PyTuple_SetItem (item, 6,
-		       _cubrid_return_PyString_FromString (class_name));
-      PyTuple_SetItem (item, 7,
-		       _cubrid_return_PyString_FromString (default_value));
-      PyTuple_SetItem (item, 8,
-		       _cubrid_return_PyInt_FromLong (auto_increment));
+      PyTuple_SetItem (item, 4, _cubrid_return_PyString_FromString (col_name));
+      PyTuple_SetItem (item, 5, _cubrid_return_PyString_FromString (real_attr));
+      PyTuple_SetItem (item, 6, _cubrid_return_PyString_FromString (class_name));
+      PyTuple_SetItem (item, 7, _cubrid_return_PyString_FromString (default_value));
+      PyTuple_SetItem (item, 8, _cubrid_return_PyInt_FromLong (auto_increment));
       PyTuple_SetItem (item, 9, _cubrid_return_PyInt_FromLong (unique_key));
       PyTuple_SetItem (item, 10, _cubrid_return_PyInt_FromLong (primary_key));
       PyTuple_SetItem (item, 11, _cubrid_return_PyInt_FromLong (foreign_key));
-      PyTuple_SetItem (item, 12,
-		       _cubrid_return_PyInt_FromLong (reverse_index));
-      PyTuple_SetItem (item, 13,
-		       _cubrid_return_PyInt_FromLong (resverse_unique));
+      PyTuple_SetItem (item, 12, _cubrid_return_PyInt_FromLong (reverse_index));
+      PyTuple_SetItem (item, 13, _cubrid_return_PyInt_FromLong (resverse_unique));
       PyTuple_SetItem (item, 14, _cubrid_return_PyInt_FromLong (shared));
 
       PyTuple_SetItem (result, j, item);
@@ -1747,14 +1850,15 @@ _cubrid_CursorObject_result_info (_cubrid_CursorObject * self,
 
 static char _cubrid_CursorObject_execute__doc__[] =
 "execute([option[,max_col_size]])\n\
-Executes a prepared Query.\n\
-A option can be used when retrieving the query result from the server.\n\
-A option can be classified as synchronous or asynchronous. \n\
-If the option is set to CUBRID_EXEC_QUERY_ALL, a synchronous mode(sync_mode) \n\
-is used to retrieve query results immediately after executing prepared queries. \n\
-If it is set to CUBRID_EXEC_ASYNC, an asynchronous mode (async_mode) is used to\n\
+execute the prepared SQL statement, which is executing prepare() and\n\
+bind_param()/bind_lob(). The function of retrieving the query result\n\
+from the server through a flag can be classified as synchronous or\n\
+asynchronous. If the flag is set to CUBRID_EXEC_QUERY_ALL, a\n\
+synchronous mode(sync_mode) is used to retrieve query results\n\
+immediately after executing prepared queries. If it is set to\n\
+CUBRID_EXEC_ASYNC, an asynchronous mode (async_mode) is used to\n\
 retrieve the result immediately each time a query result is created.\n\
-The option is set to CUBRID_EXEC_QUERY_ALL by default, and in such\n\
+The flag is set to CUBRID_EXEC_QUERY_ALL by default, and in such\n\
 cases the following rules are applied:\n\
   - The return value is the result of the first query.\n\
   - If an error occurs in any query, the execution is processed\n\
@@ -1771,7 +1875,7 @@ prepared query is CHAR, VARCHAR, NCHAR, VARNCHAR, BIT or VARBIT.\n\
 If it is set to 0, all data is transferred.\n\
 \n\
 Parameters::\n\
-  option: Exec option, option maybe the following values:\n\
+  flag: Exec flag, flag maybe the following values:\n\
     CUBRID_EXEC_ASYNC\n\
     CUBRID_EXEC_QUERY_ALL\n\
     CUBRID_EXEC_QUERY_INFO\n\
@@ -1846,11 +1950,6 @@ _cubrid_CursorObject_execute (_cubrid_CursorObject * self, PyObject * args)
 * 
 * int, short 			-> Integer
 * float, double, numeric 	-> Float
-* numeric   			-> Decimal
-* time 					-> datetime.time
-* date 					-> datetime.date
-* datetime 				-> datetime.datetime
-* timestamp 			-> datetime.datetime
 * another type			-> String
 */
 
@@ -1862,7 +1961,6 @@ _cubrid_CursorObject_dbval_to_pyvalue (_cubrid_CursorObject * self, int type,
   PyObject *val, *tmpval;
   char *buffer;
   int num;
-  T_CCI_DATE dt;
 
   switch (type)
     {
@@ -1885,6 +1983,7 @@ _cubrid_CursorObject_dbval_to_pyvalue (_cubrid_CursorObject * self, int type,
       break;
     case CCI_U_TYPE_FLOAT:
     case CCI_U_TYPE_DOUBLE:
+    case CCI_U_TYPE_NUMERIC:
       res = cci_get_data (self->handle, index, CCI_A_TYPE_STR, &buffer, &ind);
       if (res < 0)
 	{
@@ -1905,90 +2004,7 @@ _cubrid_CursorObject_dbval_to_pyvalue (_cubrid_CursorObject * self, int type,
 #endif
 	  Py_DECREF (tmpval);
 	}
-	  break;
-    case CCI_U_TYPE_NUMERIC:
-	  res = cci_get_data (self->handle, index, CCI_A_TYPE_STR, &buffer, &ind);
-	  if (res < 0) 
-	{
-	   return handle_error (res, NULL);
-	}
-	  if (ind < 0) 
-	{
-	  Py_INCREF (Py_None);
-	  val = Py_None;
-	}
-	  else
-	{
-	  tmpval = PyTuple_New (1);
-	  PyTuple_SetItem (tmpval, 0, Py_BuildValue ("s", buffer));
-	  val = PyObject_CallObject(_func_Decimal, tmpval);
-	  Py_DECREF (tmpval);
-	}
-	  break;
-    case CCI_U_TYPE_DATE:
-	  res = cci_get_data (self->handle, index, CCI_A_TYPE_DATE, &dt, &ind);
-	  if (res < 0)
-	{
-	  return handle_error (res, NULL);
-	}
-	  if (ind < 0)
-	{
-	  Py_INCREF (Py_None);
-	  val = Py_None;
-	}
-	  else
-	{
-	  val = PyDate_FromDate (dt.yr, dt.mon, dt.day);
-	}
-	  break;
-    case CCI_U_TYPE_TIME:
-	  res = cci_get_data (self->handle, index, CCI_A_TYPE_DATE, &dt, &ind);
-	  if (res < 0)
-	{
-	  return handle_error (res, NULL);
-	}
-	  if (ind < 0)
-	{
-	  Py_INCREF (Py_None);
-	  val = Py_None;
-	}
-	  else
-	{
-	  val = PyTime_FromTime (dt.hh, dt.mm, dt.ss, 0);
-	}
-	  break;
-    case CCI_U_TYPE_DATETIME:
-	  res = cci_get_data (self->handle, index, CCI_A_TYPE_DATE, &dt, &ind);
-	  if (res < 0)
-	{
-	  return handle_error (res, NULL);
-	}
-	  if (ind < 0)
-	{
-	  Py_INCREF (Py_None);
-	  val = Py_None;
-	}
-	  else
-	{
-	  val = PyDateTime_FromDateAndTime (dt.yr, dt.mon, dt.day, dt.hh, dt.mm, dt.ss, dt.ms * 1000);
-	}
-	  break;
-    case CCI_U_TYPE_TIMESTAMP:
-	  res = cci_get_data (self->handle, index, CCI_A_TYPE_DATE, &dt, &ind);
-	  if (res < 0)
-	{
-	  return handle_error (res, NULL);
-	}
-	  if (ind < 0)
-	{
-	  Py_INCREF (Py_None);
-	  val = Py_None;
-	}
-	  else
-	{
-	  val = PyDateTime_FromDateAndTime (dt.yr, dt.mon, dt.day, dt.hh, dt.mm, dt.ss, 0);
-	}
-	  break;
+      break;
     default:
       res = cci_get_data (self->handle, index, CCI_A_TYPE_STR, &buffer, &ind);
       if (res < 0)
@@ -2002,14 +2018,7 @@ _cubrid_CursorObject_dbval_to_pyvalue (_cubrid_CursorObject * self, int type,
 	}
       else
 	{
-          if (self->charset != NULL && *(self->charset) != '\0')
-            {
-              val = _cubrid_return_PyUnicode_FromString (buffer, strlen(buffer), self->charset, NULL);
-            }
-          else
-            {
-              val = _cubrid_return_PyString_FromString (buffer);
-            }
+	  val = _cubrid_return_PyString_FromString (buffer);
 	}
       break;
     }
@@ -2063,7 +2072,7 @@ _cubrid_CursorObject_dbset_to_pyvalue (_cubrid_CursorObject * self, int index)
 }
 
 static PyObject *
-_cubrid_CursorObject_fetch_row (_cubrid_CursorObject * self)
+_cubrid_row_to_tuple (_cubrid_CursorObject * self)
 {
   int i, type;
   PyObject *row, *val;
@@ -2088,6 +2097,37 @@ _cubrid_CursorObject_fetch_row (_cubrid_CursorObject * self)
   return row;
 }
 
+static PyObject *
+_cubrid_row_to_dict (_cubrid_CursorObject * self)
+{
+  PyObject *row, *val;
+  int i, type;
+
+  if (!(row = PyDict_New ()))
+    {
+      return NULL;
+    }
+
+  for (i = 0; i < self->col_count; i++)
+    {
+      char *col_name = CCI_GET_RESULT_INFO_NAME(self->col_info, i + 1);
+
+      type = CCI_GET_RESULT_INFO_TYPE (self->col_info, i + 1);
+
+      if (CCI_IS_COLLECTION_TYPE (type))
+	{
+	  val = _cubrid_CursorObject_dbset_to_pyvalue (self, i + 1);
+	}
+      else
+	{
+	  val = _cubrid_CursorObject_dbval_to_pyvalue (self, type, i + 1);
+	}
+      PyMapping_SetItemString(row, col_name, val);
+    }
+
+  return row;
+}
+
 static char _cubrid_CursorObject_fetch__doc__[] =
 "fetch_row()\n\
 get a single row from the query result. The cursor automatically moves\n\
@@ -2095,7 +2135,7 @@ to the next row after getting the result.\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect('CUBRID:localhost:33000:demodb:::', 'public')\n\
+  con = _cubrid.connect('CUBRID:localhost:33000:demodb', 'public')\n\
   cur = con.cursor()\n\
   cur.prepare('select * from test_cubrid')\n\
   cur.execute()\n\
@@ -2109,15 +2149,20 @@ Example::\n\
 static PyObject *
 _cubrid_CursorObject_fetch (_cubrid_CursorObject * self, PyObject * args)
 {
-  int res;
+  int res, how = 0;
   T_CCI_ERROR error;
   PyObject *row;
 
-  if (!PyArg_ParseTuple (args, ""))
+  if (!PyArg_ParseTuple (args, "|i", &how))
     {
       return NULL;
     }
 
+  if (how < 0 || how > 1)
+    {
+      return handle_error(CUBRID_ER_INVALID_PARAM, NULL);
+    }
+
   res = cci_cursor (self->handle, 0, CCI_CURSOR_CURRENT, &error);
   if (res == CCI_ER_NO_MORE_DATA)
     {
@@ -2135,8 +2180,14 @@ _cubrid_CursorObject_fetch (_cubrid_CursorObject * self, PyObject * args)
       return handle_error (res, &error);
     }
 
-
-  row = _cubrid_CursorObject_fetch_row (self);
+  if (how == 0)
+    {
+      row = _cubrid_row_to_tuple (self);
+    }
+  else
+    {
+      row = _cubrid_row_to_dict (self);
+    }
 
   res = cci_cursor (self->handle, 1, CCI_CURSOR_CURRENT, &error);
   if (res < 0 && res != CCI_ER_NO_MORE_DATA)
@@ -2160,7 +2211,7 @@ Parameters::\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect('CUBRID:localhost:33000:demodb:::', 'public')\n\
+  con = _cubrid.connect('CUBRID:localhost:33000:demodb', 'public')\n\
   cur = con.cursor()\n\
   cur.prepare('select * from test_lob')\n\
   cur.execute()\n\
@@ -2685,7 +2736,7 @@ It will be create a BLOB object as default.\n\
 \n\
 Example 1::\n\
   import _cubrid\n\
-  con = _cubrid.connect('CUBRID:localhost:33000:demodb:::', 'public')\n\
+  con = _cubrid.connect('CUBRID:localhost:33000:demodb', 'public')\n\
   cur = con.cursor()\n\
   cur.prepare('insert into test_clob(content) values (?)')\n\
   lob = con.lob()\n\
@@ -2699,7 +2750,7 @@ Example 1::\n\
 \n\
 Example 2::\n\
   import _cubrid\n\
-  con = _cubrid.connect('CUBRID:localhost:33000:demodb:::', 'public')\n\
+  con = _cubrid.connect('CUBRID:localhost:33000:demodb', 'public')\n\
   cur = con.cursor()\n\
   cur.prepare('select * from test_blob')\n\
   cur.execute()\n\
@@ -2771,7 +2822,7 @@ file: string, support filepath/file\n\
 \n\
 Example::\n\
   import _cubrid\n\
-  con = _cubrid.connect('CUBRID:localhost:33000:demodb:::', 'public')\n\
+  con = _cubrid.connect('CUBRID:localhost:33000:demodb', 'public')\n\
   cur = con.cursor()\n\
   cur.prepare('select * from test_lob')\n\
   cur.execute()\n\
@@ -2847,7 +2898,7 @@ Return a string that contains the data read.\n\
 \n\
 Example 1::\n\
   import _cubrid\n\
-  con = _cubrid.connect('CUBRID:localhost:33000:demodb:::', 'public')\n\
+  con = _cubrid.connect('CUBRID:localhost:33000:demodb', 'public')\n\
   lob = con.lob()\n\
   lob.imports('README', 'C')\n\
   str = lob.read(32)\n\
@@ -2857,7 +2908,7 @@ Example 1::\n\
 \n\
 Example 2::\n\
   import _cubrid\n\
-  con = _cubrid.connect('CUBRID:localhost:33000:demodb:::', 'public')\n\
+  con = _cubrid.connect('CUBRID:localhost:33000:demodb', 'public')\n\
   cur = con.cursor()\n\
   cur.prepare('select * from test_lob')\n\
   cur.execute()\n\
@@ -3075,11 +3126,6 @@ static PyMethodDef _cubrid_CursorObject_methods[] = {
    (PyCFunction) _cubrid_CursorObject_prepare,
    METH_VARARGS,
    _cubrid_CursorObject_prepare__doc__},
-  {
-   "_set_charset_name",
-   (PyCFunction) _cubrid_CursorObject__set_charset_name,
-   METH_VARARGS,
-   _cubrid_CursorObject__set_charset_name__doc__},
   {
    "bind_param",
    (PyCFunction) _cubrid_CursorObject_bind_param,
@@ -3398,7 +3444,7 @@ PyTypeObject _cubrid_CursorObject_type = {
   0,				/* tp_free */
 };
 
-#define _CUBRID_VERSION_	"8.4.3.0004"
+#define _CUBRID_VERSION_	"9.0.0.0001"
 static char _cubrid_doc[] = "CUBRID API Module for Python";
 
 #if PY_MAJOR_VERSION >= 3
@@ -3422,6 +3468,10 @@ init_exceptions (PyObject * dict)
     PyErr_NewException ("_cubrid.Error", PyExc_Exception, NULL);
   PyDict_SetItemString (dict, "Error", _cubrid_error);
 
+  _cubrid_warning =
+    PyErr_NewException ("_cubrid.Warning", PyExc_Exception, NULL);
+  PyDict_SetItemString (dict, "Warning", _cubrid_warning);
+
   _cubrid_interface_error =
     PyErr_NewException ("_cubrid.InterfaceError", _cubrid_error, NULL);
   PyDict_SetItemString (dict, "InterfaceError", _cubrid_interface_error);
@@ -3430,9 +3480,35 @@ init_exceptions (PyObject * dict)
     PyErr_NewException ("_cubrid.DatabaseError", _cubrid_error, NULL);
   PyDict_SetItemString (dict, "DatabaseError", _cubrid_database_error);
 
+  _cubrid_internal_error =
+    PyErr_NewException ("_cubrid.InternalError", _cubrid_database_error,
+			NULL);
+  PyDict_SetItemString (dict, "InternalError", _cubrid_internal_error);
+
+  _cubrid_operational_error =
+    PyErr_NewException ("_cubrid.OperationalError", _cubrid_database_error,
+			NULL);
+  PyDict_SetItemString (dict, "OperationalError", _cubrid_operational_error);
+
+  _cubrid_programming_error =
+    PyErr_NewException ("_cubrid.ProgrammingError", _cubrid_database_error,
+			NULL);
+  PyDict_SetItemString (dict, "ProgrammingError", _cubrid_programming_error);
+
+  _cubrid_integrity_error =
+    PyErr_NewException ("_cubrid.IntegrityError", _cubrid_database_error,
+			NULL);
+  PyDict_SetItemString (dict, "IntegrityError", _cubrid_integrity_error);
+
+  _cubrid_data_error =
+    PyErr_NewException ("_cubrid.DataError", _cubrid_database_error, NULL);
+  PyDict_SetItemString (dict, "DataError", _cubrid_data_error);
+
   _cubrid_not_supported_error =
-    PyErr_NewException ("_cubrid.NotSupportedError", _cubrid_database_error,NULL);
-  PyDict_SetItemString (dict, "NotSupportedError", _cubrid_not_supported_error);
+    PyErr_NewException ("_cubrid.NotSupportedError", _cubrid_database_error,
+			NULL);
+  PyDict_SetItemString (dict, "NotSupportedError",
+			_cubrid_not_supported_error);
 }
 
 static int
@@ -3573,7 +3649,7 @@ void
 init_cubrid (void)
 #endif
 {
-  PyObject *dict, *module, *mDecimal;
+  PyObject *dict, *module;
 
 #if PY_MAJOR_VERSION >= 3
   module = PyModule_Create (&cubriddef);
@@ -3630,25 +3706,6 @@ init_cubrid (void)
       goto Error;
     }
 
- /* import function Decimal from module decimal */
-  mDecimal = PyImport_ImportModule ("decimal");
-  if (!mDecimal)
-    {
-      goto Error;
-    }
-
-  _func_Decimal = PyObject_GetAttrString (mDecimal, "Decimal");
-  if (!_func_Decimal)
-    {
-      goto Error;
-    }
-
-  Py_INCREF (_func_Decimal);
-  Py_DECREF (mDecimal);
-
-  /* invoke PyDateTime_IMPORT macro to use functions from datetime.h */
-  PyDateTime_IMPORT;
-
 #if PY_MAJOR_VERSION >= 3
   return module;
 #else
